# -*- coding: utf-8 -*-
from setuphelpers import *
import time

discord_install_dir = makepath(programfiles32, "discord_install")
discord_install_path = makepath(discord_install_dir, "DiscordSetup.exe")


def install():
    mkdirs(discord_install_dir)
    filecopyto("DiscordSetup.exe", discord_install_path)
    discord_squirrel_remove()


def session_setup():
    # Remove programdata installs
    programdata_user_dir = makepath(programdata, get_current_user)
    programdata_app_dir = makepath(programdata_user_dir, "Discord")
    programdata_squirrel_dir = makepath(programdata, get_current_user, "SquirrelTemp")

    if isdir(programdata_app_dir):
        if not isfile(makepath(programdata_app_dir, ".dead")):
            try:
                app_uninstall_cmd = rf'"{programdata_app_dir}\Update.exe" --uninstall -s'
                run(app_uninstall_cmd)
            except:
                print("Unable to remove Discord in: %s" % programdata_app_dir)
        remove_tree(programdata_app_dir)

    if isdir(programdata_squirrel_dir):
        remove_tree(programdata_squirrel_dir)
    if dir_is_empty(programdata_user_dir):
        remove_tree(programdata_user_dir)

    # Install app
    user_app_dir = makepath(user_local_appdata, "Discord")
    discord_versions = glob.glob(makepath(user_app_dir, "**/Discord.exe"))
    if discord_versions:
        discord_version = "1.0"
        for d in discord_versions:
            if Version(get_file_properties(d)["ProductVersion"]) > Version(discord_version):
                discord_version = get_file_properties(d)["ProductVersion"]
        if Version(control.get_software_version()) > Version(discord_version):
            run('"%s"' % discord_install_path)
        else:
            print("Discord already up-to-date (%s)" % discord_version)
    else:
        run('"%s"' % discord_install_path)

    # Disable app run at startup
    reg_app_run = None
    loop = 0
    loop_timer = 1
    max_loop = 120
    while not reg_app_run:
        reg_app_run = registry_readstring(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Discord")
        loop += loop_timer
        if loop >= max_loop:
            break
        time.sleep(loop_timer)
    if reg_app_run:
        registry_setstring(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run\AutorunsDisabled", "Discord", reg_app_run)
        registry_delete(HKEY_CURRENT_USER, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Discord")

    # Discord interact differently if user is admin
    if running_as_admin():
        discord_squirrel_remove()


def uninstall():
    if isdir(discord_install_dir):
        remove_tree(discord_install_dir)
    discord_squirrel_remove()
    registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\StartupApproved\Run", "Discord")
    registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\StartupApproved\Run32", "Discord")

    # if discord was uninstalled a .dead file is left behind to prevent automatic reinstall
    for user_dir in glob.glob(makepath(systemdrive, "Users", "*")):
        # "C:\Users\username\AppData\Local\Discord\Update.exe"
        local_appdata_app_dir = makepath(user_dir, "AppData", "Local", "Discord")
        if isdir(local_appdata_app_dir):
            if not isfile(makepath(local_appdata_app_dir, ".dead")):
                try:
                    app_uninstall_cmd = rf'"{local_appdata_app_dir}\Update.exe" --uninstall -s'
                    print(app_uninstall_cmd)
                    run(app_uninstall_cmd)
                except:
                    print("Unable to remove Discord for user: %s" % user_dir.split(os.sep)[-1])
            remove_tree(local_appdata_app_dir)

        # "C:\Users\jpadmin\AppData\Roaming\Microsoft\Windows\Start Menu\Programs\Discord Inc"
        start_shortcut = makepath(user_dir, r"AppData\Roaming\Microsoft\Windows\Start Menu\Programs\Discord Inc")
        desktop_shortcut = makepath(user_dir, r"Desktop\Discord.lnk")
        if isfile(desktop_shortcut):
            remove_file(desktop_shortcut)
        if isdir(start_shortcut):
            remove_tree(start_shortcut)


def discord_squirrel_remove():
    squirrel_dir = makepath(programdata, "SquirrelMachineInstalls")
    discord_old_installer = makepath(programdata, "Discord.exe")
    if isfile(discord_old_installer):
        remove_file(discord_old_installer)
    if dir_is_empty(squirrel_dir):
        remove_tree(squirrel_dir)

    registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\Microsoft\Windows\CurrentVersion\Run", "Discord")
    registry_delete(HKEY_LOCAL_MACHINE, r"SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Run", "Discord")
